#!/usr/bin/perl

=for comment

COPYRIGHT NOTICE:

This software, a Perl package named DBIx::Connect, 
is released under the same copyright terms as Perl itself.

=cut


use DBIx::Connect;

$ENV{DBIX_CONN} = "complex_conf";


